@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: dbdriver_message.c,v 1.6 2006/06/26 11:50:26 hoa Exp $
+ * $Id: dbdriver_message.c,v 1.8 2008/02/17 13:13:26 hoa Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,7 +45,11 @@
 #include "generic_cache.h"
 
 #include <unistd.h>
-#include <sys/mman.h>
+#ifdef WIN32
+#	include "win_etpan.h"
+#else
+#	include <sys/mman.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -67,7 +71,6 @@ static int initialize(mailmessage * msg_info);
 static void check(mailmessage * msg_info);
 
 static mailmessage_driver local_db_message_driver = {
-#if ! defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
   /* msg_name */ "db",
 
   /* msg_initialize */ initialize,
@@ -89,31 +92,7 @@ static mailmessage_driver local_db_message_driver = {
   /* msg_fetch_section_body */ mailmessage_generic_fetch_section_body,
   /* msg_fetch_envelope */ fetch_envelope,
 
-  /* msg_get_flags */ get_flags,
-#else
-  .msg_name = "db",
-
-  .msg_initialize = initialize,
-  .msg_uninitialize = mailmessage_generic_uninitialize,
-
-  .msg_flush = mailmessage_generic_flush,
-  .msg_check = check,
-
-  .msg_fetch_result_free = mailmessage_generic_fetch_result_free,
-
-  .msg_fetch = mailmessage_generic_fetch,
-  .msg_fetch_header = mailmessage_generic_fetch_header,
-  .msg_fetch_body = mailmessage_generic_fetch_header,
-  .msg_fetch_size = NULL,
-  .msg_get_bodystructure = mailmessage_generic_get_bodystructure,
-  .msg_fetch_section = mailmessage_generic_fetch_section,
-  .msg_fetch_section_header = mailmessage_generic_fetch_section_header,
-  .msg_fetch_section_mime = mailmessage_generic_fetch_section_mime,
-  .msg_fetch_section_body = mailmessage_generic_fetch_section_body,
-  .msg_fetch_envelope = fetch_envelope,
-
-  .msg_get_flags = get_flags,
-#endif
+  /* msg_get_flags */ get_flags
 };
 
 mailmessage_driver * db_message_driver = &local_db_message_driver;
