@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailimap.h,v 1.18 2006/10/20 00:13:30 hoa Exp $
+ * $Id: mailimap.h,v 1.23 2011/03/29 23:59:05 hoa Exp $
  */
 
 #ifndef MAILIMAP_H
@@ -51,6 +51,10 @@ extern "C" {
 #include <libetpan/acl.h>
 #include <libetpan/annotatemore.h>
 #include <libetpan/uidplus.h>
+#include <libetpan/idle.h>
+#include <libetpan/quota.h>
+#include <libetpan/namespace.h>
+#include <libetpan/xlist.h>
 
 /*
   mailimap_connect()
@@ -644,12 +648,18 @@ mailimap * mailimap_new(size_t imap_progr_rate,
 LIBETPAN_EXPORT
 void mailimap_free(mailimap * session);
 
-int send_current_tag(mailimap * session);
+int mailimap_send_current_tag(mailimap * session);
 
-char * read_line(mailimap * session);
+char * mailimap_read_line(mailimap * session);
 
-int parse_response(mailimap * session,
-			   struct mailimap_response ** result);
+int mailimap_parse_response(mailimap * session,
+    struct mailimap_response ** result);
+
+LIBETPAN_EXPORT
+void mailimap_set_progress_callback(mailimap * session,
+                                    mailprogress_function * body_progr_fun,
+                                    mailprogress_function * items_progr_fun,
+                                    void * context);
 
 #ifdef __cplusplus
 }
