@@ -44,6 +44,8 @@
 #include "acl.h"
 #include "uidplus.h"
 #include "quota.h"
+#include "namespace.h"
+#include "xlist.h"
 
 /*
   the list of registered extensions (struct mailimap_extension_api *)
@@ -58,6 +60,8 @@ static struct mailimap_extension_api * internal_extension_list[] = {
   &mailimap_extension_acl,
   &mailimap_extension_uidplus,
   &mailimap_extension_quota,
+  &mailimap_extension_namespace,
+  &mailimap_extension_xlist,
 };
 
 LIBETPAN_EXPORT
