@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailimap_parser.h,v 1.14 2010/01/03 22:45:24 hoa Exp $
+ * $Id: mailimap_parser.h,v 1.17 2011/03/30 13:29:49 hoa Exp $
  */
 
 #ifndef MAILIMAP_PARSER_H
@@ -55,6 +55,13 @@ mailimap_response_parse(mailstream * fd, MMAPString * buffer,
 			size_t progr_rate,
 			progress_function * progr_fun);
 
+int
+mailimap_response_parse_with_context(mailstream * fd, MMAPString * buffer,
+                                     size_t * indx, struct mailimap_response ** result,
+                                     mailprogress_function * body_progress_fun,
+                                     mailprogress_function * items_progress_fun,
+                                     void * context);
+
 int
 mailimap_continue_req_parse(mailstream * fd, MMAPString * buffer,
 			    size_t * indx,
@@ -73,6 +80,7 @@ typedef int mailimap_struct_parser(mailstream * fd, MMAPString * buffer,
 				   size_t * indx, void * result,
 				   size_t progr_rate,
 				   progress_function * progr_fun);
+
 typedef void mailimap_struct_destructor(void * result);
 
 int
@@ -80,7 +88,13 @@ mailimap_mailbox_parse(mailstream * fd, MMAPString * buffer,
 		       size_t * indx, char ** result,
 		       size_t progr_rate,
 		       progress_function * progr_fun);
-
+int
+mailimap_mailbox_list_parse(mailstream * fd, MMAPString * buffer,
+                            size_t * indx,
+                            struct mailimap_mailbox_list ** result,
+                            size_t progr_rate,
+                            progress_function * progr_fun);
+  
 int mailimap_nstring_parse(mailstream * fd, MMAPString * buffer,
 				  size_t * indx, char ** result,
 				  size_t * result_len,
@@ -143,6 +157,16 @@ int mailimap_uniqueid_parse(mailstream * fd, MMAPString * buffer,
 int mailimap_colon_parse(mailstream * fd, MMAPString * buffer,
     size_t * indx);
 
+int mailimap_dquote_parse(mailstream * fd, MMAPString * buffer,
+                          size_t * indx);
+
+int
+mailimap_quoted_char_parse(mailstream * fd, MMAPString * buffer,
+                           size_t * indx, char * result);
+  
+int mailimap_nil_parse(mailstream * fd, MMAPString * buffer,
+                       size_t * indx);
+
 int
 mailimap_struct_multiple_parse(mailstream * fd, MMAPString * buffer,
 			       size_t * indx, clist ** result,
