@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailsmtp_types.h,v 1.16 2006/10/13 14:24:03 alfie Exp $
+ * $Id: mailsmtp_types.h,v 1.20 2011/02/28 22:40:02 hoa Exp $
  */
 
 #ifndef MAILSMTP_TYPES_H
@@ -71,7 +71,8 @@ enum {
   MAILSMTP_ERROR_STARTTLS_TEMPORARY_FAILURE,
   MAILSMTP_ERROR_STARTTLS_NOT_SUPPORTED,
   MAILSMTP_ERROR_CONNECTION_REFUSED,
-  MAILSMTP_ERROR_AUTH_AUTHENTICATION_FAILED
+  MAILSMTP_ERROR_AUTH_AUTHENTICATION_FAILED,
+  MAILSMTP_ERROR_SSL
 };
 
 enum {
@@ -80,7 +81,11 @@ enum {
   MAILSMTP_AUTH_CRAM_MD5 = 2,
   MAILSMTP_AUTH_PLAIN = 4,
   MAILSMTP_AUTH_LOGIN = 8,
-  MAILSMTP_AUTH_DIGEST_MD5 = 16
+  MAILSMTP_AUTH_DIGEST_MD5 = 16,
+  MAILSMTP_AUTH_GSSAPI = 32,
+  MAILSMTP_AUTH_SRP = 64,
+  MAILSMTP_AUTH_NTLM = 128,
+  MAILSMTP_AUTH_KERBEROS_V4 = 256,
 };
 
 enum {
@@ -116,6 +121,11 @@ struct mailsmtp {
     const char * sasl_realm;
     void * sasl_secret;
   } smtp_sasl;
+
+  size_t smtp_max_msg_size;
+
+  mailprogress_function * smtp_progress_fun;
+  void * smtp_progress_context;
 };
 
 typedef struct mailsmtp mailsmtp;
